/** @file *//********************************************************************************************************

                                                    VertexBufferProxy.cpp

	--------------------------------------------------------------------------------------------------------------

	$Header: //depot/Libraries/Dxx/VertexBufferProxy.cpp#6 $

	$NoKeywords: $

*********************************************************************************************************************/

#include "PrecompiledHeaders.h"

#include "VertexBufferProxy.h"

#include <d3d9.h>
#include <d3dx9.h>


namespace Dxx
{


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

///
/// @param	pMesh	Mesh containing the vertex buffer to proxy

VertexBufferProxy::VertexBufferProxy( ID3DXMesh * pMesh )
{
	pMesh->GetVertexBuffer( &m_pBuffer );

	// Get the "Desc" data

	m_pBuffer->GetDesc( &m_Desc );

	InitializeOffsets();
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

/// @param	pBuffer		Vertex buffer to proxy

VertexBufferProxy::VertexBufferProxy( IDirect3DVertexBuffer9 * pBuffer )
	: m_pBuffer( pBuffer )
{
	m_pBuffer->AddRef();

	// Get the "Desc" data

	m_pBuffer->GetDesc( &m_Desc );

	InitializeOffsets();
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

void VertexBufferProxy::InitializeOffsets()
{
	int		bump;

	// Compute the stride and offset values

	switch ( m_Desc.FVF & D3DFVF_POSITION_MASK )
	{
	case D3DFVF_XYZ:
		bump					= sizeof( D3DVECTOR );
		break;

	case D3DFVF_XYZRHW:
	case D3DFVF_XYZW:
		m_WOffset				= sizeof( D3DVECTOR );
		bump					= m_WOffset				+ sizeof( float );
		break;

	case D3DFVF_XYZB1:
		m_Blend0Offset			= sizeof( D3DVECTOR );
		bump					= m_Blend0Offset		+ sizeof( float );
		break;

	case D3DFVF_XYZB2:
		m_Blend0Offset			= sizeof( D3DVECTOR );
		m_Blend1Offset			= m_Blend0Offset 		+ sizeof( float );			
		bump					= m_Blend1Offset 		+ sizeof( float );
		break;

	case D3DFVF_XYZB3:
		m_Blend0Offset			= sizeof( D3DVECTOR );
		m_Blend1Offset			= m_Blend0Offset 		+ sizeof( float );			
		m_Blend2Offset			= m_Blend1Offset 		+ sizeof( float );			
		bump					= m_Blend2Offset 		+ sizeof( float );
		break;

	case D3DFVF_XYZB4:
		m_Blend0Offset			= sizeof( D3DVECTOR );
		m_Blend1Offset			= m_Blend0Offset 		+ sizeof( float );			
		m_Blend2Offset			= m_Blend1Offset 		+ sizeof( float );			
		m_Blend3Offset			= m_Blend2Offset 		+ sizeof( float );			
		bump					= m_Blend3Offset 		+ sizeof( float );
		break;

	case D3DFVF_XYZB5:
		m_Blend0Offset			= sizeof( D3DVECTOR );
		m_Blend1Offset			= m_Blend0Offset		+ sizeof( float );			
		m_Blend2Offset			= m_Blend1Offset		+ sizeof( float );			
		m_Blend3Offset			= m_Blend2Offset		+ sizeof( float );			
		m_BlendIndexesOffset	= m_Blend3Offset		+ sizeof( float );	
		bump					= m_BlendIndexesOffset	+ sizeof( DWORD );
		break;

	default:
		bump					 = 0;	// No position value.
		break;
	}

	m_NormalOffset			= bump;			
	m_PointSizeOffset		= bump;		
	m_DiffuseOffset			= bump;		
	m_SpecularOffset		= bump;		
	m_TexCoord0Offset		= bump;		
	m_TexCoord1Offset		= bump;		
	m_TexCoord2Offset		= bump;		
	m_TexCoord3Offset		= bump;		
	m_TexCoord4Offset		= bump;		
	m_TexCoord5Offset		= bump;		
	m_TexCoord6Offset		= bump;		
	m_TexCoord7Offset		= bump;
	m_Stride				= bump;

	if ( ( m_Desc.FVF & D3DFVF_NORMAL ) != 0 )
	{
		bump					= sizeof( D3DVECTOR );
		m_PointSizeOffset		+= bump;
		m_DiffuseOffset			+= bump;		
		m_SpecularOffset		+= bump;		
		m_TexCoord0Offset		+= bump;		
		m_TexCoord1Offset		+= bump;		
		m_TexCoord2Offset		+= bump;		
		m_TexCoord3Offset		+= bump;		
		m_TexCoord4Offset		+= bump;		
		m_TexCoord5Offset		+= bump;		
		m_TexCoord6Offset		+= bump;		
		m_TexCoord7Offset		+= bump;
		m_Stride				+= bump;
	}

	if ( ( m_Desc.FVF & D3DFVF_PSIZE ) != 0 )
	{
		bump					= sizeof( float );
		m_DiffuseOffset			+= bump;		
		m_SpecularOffset		+= bump;		
		m_TexCoord0Offset		+= bump;		
		m_TexCoord1Offset		+= bump;		
		m_TexCoord2Offset		+= bump;		
		m_TexCoord3Offset		+= bump;		
		m_TexCoord4Offset		+= bump;		
		m_TexCoord5Offset		+= bump;		
		m_TexCoord6Offset		+= bump;		
		m_TexCoord7Offset		+= bump;
		m_Stride				+= bump;
	}

	if ( ( m_Desc.FVF & D3DFVF_DIFFUSE ) != 0 )
	{
		bump					= sizeof( DWORD );
		m_SpecularOffset		+= bump;		
		m_TexCoord0Offset		+= bump;		
		m_TexCoord1Offset		+= bump;		
		m_TexCoord2Offset		+= bump;		
		m_TexCoord3Offset		+= bump;		
		m_TexCoord4Offset		+= bump;		
		m_TexCoord5Offset		+= bump;		
		m_TexCoord6Offset		+= bump;		
		m_TexCoord7Offset		+= bump;
		m_Stride				+= bump;
	}

	if ( ( m_Desc.FVF & D3DFVF_SPECULAR ) != 0 )
	{
		bump					= sizeof( DWORD );
		m_TexCoord0Offset		+= bump;		
		m_TexCoord1Offset		+= bump;		
		m_TexCoord2Offset		+= bump;		
		m_TexCoord3Offset		+= bump;		
		m_TexCoord4Offset		+= bump;		
		m_TexCoord5Offset		+= bump;		
		m_TexCoord6Offset		+= bump;		
		m_TexCoord7Offset		+= bump;
		m_Stride				+= bump;
	}

	switch ( m_Desc.FVF & D3DFVF_TEXCOUNT_MASK )
	{
	case D3DFVF_TEX8:
		bump = ( ( ( ( m_Desc.FVF >> 30 ) + 1 ) & 3 ) + 1 ) * sizeof( float );
		m_Stride				+= bump;
		/* fall through */

	case D3DFVF_TEX7:
		bump = ( ( ( ( m_Desc.FVF >> 28 ) + 1 ) & 3 ) + 1 ) * sizeof( float );
		m_TexCoord7Offset		+= bump;
		m_Stride				+= bump;
		/* fall through */

	case D3DFVF_TEX6:
		bump = ( ( ( ( m_Desc.FVF >> 26 ) + 1 ) & 3 ) + 1 ) * sizeof( float );
		m_TexCoord6Offset		+= bump;		
		m_TexCoord7Offset		+= bump;
		m_Stride				+= bump;
		/* fall through */

	case D3DFVF_TEX5:
		bump = ( ( ( ( m_Desc.FVF >> 24 ) + 1 ) & 3 ) + 1 ) * sizeof( float );
		m_TexCoord5Offset		+= bump;		
		m_TexCoord6Offset		+= bump;		
		m_TexCoord7Offset		+= bump;
		m_Stride				+= bump;
		/* fall through */

	case D3DFVF_TEX4:
		bump = ( ( ( ( m_Desc.FVF >> 22 ) + 1 ) & 3 ) + 1 ) * sizeof( float );
		m_TexCoord4Offset		+= bump;
		m_TexCoord5Offset		+= bump;
		m_TexCoord6Offset		+= bump;
		m_TexCoord7Offset		+= bump;
		m_Stride				+= bump;
		/* fall through */

	case D3DFVF_TEX3:
		bump = ( ( ( ( m_Desc.FVF >> 20 ) + 1 ) & 3 ) + 1 ) * sizeof( float );
		m_TexCoord3Offset		+= bump;
		m_TexCoord4Offset		+= bump;
		m_TexCoord5Offset		+= bump;
		m_TexCoord6Offset		+= bump;
		m_TexCoord7Offset		+= bump;
		m_Stride				+= bump;
		/* fall through */

	case D3DFVF_TEX2:
		bump = ( ( ( ( m_Desc.FVF >> 18 ) + 1 ) & 3 ) + 1 ) * sizeof( float );
		m_TexCoord2Offset		+= bump;
		m_TexCoord3Offset		+= bump;
		m_TexCoord4Offset		+= bump;
		m_TexCoord5Offset		+= bump;
		m_TexCoord6Offset		+= bump;
		m_TexCoord7Offset		+= bump;
		m_Stride				+= bump;
		/* fall through */

	case D3DFVF_TEX1:
		bump = ( ( ( ( m_Desc.FVF >> 16 ) + 1 ) & 3 ) + 1 ) * sizeof( float );
		m_TexCoord1Offset		+= bump;
		m_TexCoord2Offset		+= bump;
		m_TexCoord3Offset		+= bump;
		m_TexCoord4Offset		+= bump;
		m_TexCoord5Offset		+= bump;
		m_TexCoord6Offset		+= bump;
		m_TexCoord7Offset		+= bump;
		m_Stride				+= bump;
	}
}


} // namespace Dxx
